@@ -1,4 +1,4 @@
-/* KDevelop xUnit plugin
+/* KDevelop coverage plugin
  *    Copyright 2008 Manuel Breugelmans <mbr.nxi@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -39,7 +39,14 @@ void ReportFileItemTest::init()
 
 void ReportFileItemTest::cleanup()
 {
-    if (m_file) delete m_file;
+    if (m_file) {
+        delete m_file->coverageRatioItem();
+        delete m_file->nrofCoveredLinesItem();
+        delete m_file->slocItem();
+        delete m_file;
+    }
+    qDeleteAll(m_garbage);
+    m_garbage.clear();
 }
 
 void ReportFileItemTest::construct()
@@ -54,8 +61,8 @@ void ReportFileItemTest::construct()
 
     // Neighbour items should be initialized on 0
     KOMPARE(0, m_file->slocItem());
-    KOMPARE(0, m_file->coverageItem());
-    KOMPARE(0, m_file->instrumentedItem());
+    KOMPARE(0, m_file->coverageRatioItem());
+    KOMPARE(0, m_file->nrofCoveredLinesItem());
 }
 
 void ReportFileItemTest::addCoverage()
@@ -67,11 +74,13 @@ void ReportFileItemTest::addCoverage()
     assertSlocItem(2);
     assertInstrumentedItem(1);
     assertCoverageItem(100*(double)1/2);
+    
+    m_garbage << f;
 }
 
 void ReportFileItemTest::assertInstrumentedItem(int instrumented)
 {
-    ReportValueItem* instrumentedItem = m_file->instrumentedItem();
+    ReportValueItem* instrumentedItem = m_file->nrofCoveredLinesItem();
     KOMPARE(instrumented, instrumentedItem->value());
 }
 
@@ -83,7 +92,7 @@ void ReportFileItemTest::assertSlocItem(int sloc)
 
 void ReportFileItemTest::assertCoverageItem(double coverage)
 {
-    ReportValueItem* coverageItem = m_file->coverageItem();
+    ReportValueItem* coverageItem = m_file->coverageRatioItem();
     double actual = coverageItem->value();
     KVERIFY_MSG(qAbs(coverage - actual) < 0.1,
                 QString("\nWrong coverage percentage for coverage item.\n"
@@ -105,6 +114,8 @@ void ReportFileItemTest::addMultipleCoverage()
     assertInstrumentedItem(4);
     assertSlocItem(4);
     assertCoverageItem(100.0);
+    
+    m_garbage << f << f2;
 }
 
 
