@@ -58,6 +58,7 @@
 
 #include "cppdebughelper.h"
 #include "name_visitor.h"
+#include "usebuilder.h"
 
 const Identifier& castIdentifier() {
   static Identifier id("operator{...cast...}");
@@ -202,6 +203,36 @@ void DeclarationBuilder::visitFunctionDeclaration(FunctionDefinitionAST* node)
 void DeclarationBuilder::visitInitDeclarator(InitDeclaratorAST *node)
 {
   PushValue<bool> setHasInitialize(m_declarationHasInitializer, (bool)node->initializer);
+
+  if(currentContext()->type() == DUContext::Other) {
+    //Cannot declare a a function within a code-context
+    node->declarator->parameter_is_initializer = true;
+  }else if(!m_inFunctionDefinition && node->declarator && node->declarator->parameter_declaration_clause && node->declarator->id) {
+    //Decide whether the parameter-declaration clause is valid
+    DUChainWriteLocker lock(DUChain::lock());
+    SimpleCursor pos = editor()->findPosition(node->start_token, KDevelop::EditorIntegrator::FrontEdge);
+    
+    QualifiedIdentifier id;
+    identifierForNode(node->declarator->id, id);    
+    DUContext* previous = currentContext();
+
+    DUContext* previousLast = lastContext();
+    QList<KDevelop::DUContext*> importedParentContexts = m_importedParentContexts;
+    
+    openPrefixContext(node, id, pos); //We create a temporary prefix-context to search from within the right scope
+    
+    DUContext* tempContext = currentContext();
+    node->declarator->parameter_is_initializer = !checkParameterDeclarationClause(node->declarator->parameter_declaration_clause);
+    closePrefixContext(id);
+    
+    if(tempContext != previous) {
+      delete tempContext;
+      setLastContext(previousLast);
+      m_importedParentContexts = importedParentContexts;
+    }
+    Q_ASSERT(currentContext() == previous);
+  }
+  
   DeclarationBuilderBase::visitInitDeclarator(node);
 }
 
@@ -232,23 +263,22 @@ void DeclarationBuilder::visitDeclarator (DeclaratorAST* node)
   m_collectQtFunctionSignature = !m_accessPolicyStack.isEmpty() && ((m_accessPolicyStack.top() & FunctionIsSlot) || (m_accessPolicyStack.top() & FunctionIsSignal));
   m_qtFunctionSignature = QByteArray();
   
-  ///@todo this should be solved more elegantly within parser and AST
-  if (node->parameter_declaration_clause) {
-    //Check if all parameter declarations are valid. If not, this is a misunderstood variable declaration.
-    if(!checkParameterDeclarationClause(node->parameter_declaration_clause))
-      node->parameter_declaration_clause = 0;
-  }
-  if (node->parameter_declaration_clause) {
+  if (node->parameter_declaration_clause && !node->parameter_is_initializer) {
+
+    if(m_collectQtFunctionSignature) //We need to do this just to collect the signature
+      checkParameterDeclarationClause(node->parameter_declaration_clause);
+    
     Declaration* decl = openFunctionDeclaration(node->id, node);
 
     if( !m_functionDefinedStack.isEmpty() ) {
         DUChainWriteLocker lock(DUChain::lock());
         decl->setDeclarationIsDefinition( (bool)m_functionDefinedStack.top() );
     }
-
+    
     applyFunctionSpecifiers();
   } else {
     openDefinition(node->id, node, node->id == 0);
+    node->parameter_declaration_clause = 0;
   }
 
   m_collectQtFunctionSignature = false;
@@ -1440,6 +1470,9 @@ bool DeclarationBuilder::checkParameterDeclarationClause(ParameterDeclarationCla
           if( lastTypeWasInstance() ) {
             ret = false;
             break;
+          }else if(lastType().cast<DelayedType>() && lastType().cast<DelayedType>()->kind() == DelayedType::Unresolved) {
+            //When the searched item was not found, expect it to be a non-type
+            ret = false;
           }else{
             ret = true;
             break;
@@ -1453,4 +1486,4 @@ bool DeclarationBuilder::checkParameterDeclarationClause(ParameterDeclarationCla
     setLastType(oldLastType);
 
     return ret;
-}
+}
\ No newline at end of file
