@@ -118,6 +118,7 @@ private slots:
   void testIndexedStrings();
   void testEnum();
   void testCaseUse();
+  void testConstructorOperatorUses();
   void testSizeofUse();
   void testDefinitionUse();
   void testImportStructure();
@@ -132,6 +133,8 @@ private slots:
 
   void testSymbolTableValid();
   
+  void testSeparateVariableDefinition();
+  
   void testBaseUses();
 
   void testCStruct();
