@@ -58,6 +58,8 @@
 
 // #define ifDebug(x) x
 
+const uint maxIncludeDepth = 50;
+
 QString urlsToString(const QList<KUrl>& urlList) {
   QString paths;
   foreach( const KUrl& u, urlList )
@@ -88,11 +90,28 @@ CPPParseJob * PreprocessJob::parentJob() const
     return static_cast<CPPParseJob*>(const_cast<QObject*>(parent()));
 }
 
+void PreprocessJob::foundHeaderGuard(rpp::Stream& stream, KDevelop::IndexedString guardName)
+{
+  KDevelop::DUChainWriteLocker lock(KDevelop::DUChain::lock());
+  
+  m_currentEnvironment->environmentFile()->setHeaderGuard(guardName);
+  
+  //In naive matching mode, we ignore the dependence on header-guards
+  if(Cpp::EnvironmentManager::matchingLevel() <= Cpp::EnvironmentManager::Naive)
+    m_currentEnvironment->removeString(guardName);
+}
+
 void PreprocessJob::run()
 {
+    if(!CppLanguageSupport::self())
+      return;
+  
     //If we have a parent, that parent already has locked the parse-lock
     QReadLocker lock(parentJob()->parentPreprocessor() ? 0 : parentJob()->cpp()->language()->parseLock());
   
+    if(!CppLanguageSupport::self())
+      return;
+    
     //It seems like we cannot influence the actual thread priority in thread-weaver, so for now set it here.
     //It must be low so the GUI stays fluid.
     if(QThread::currentThread())
@@ -108,12 +127,12 @@ void PreprocessJob::run()
     {
       KDevelop::DUChainReadLocker readLock(KDevelop::DUChain::lock());
       
-    if(Cpp::EnvironmentManager::isSimplifiedMatching()) {
+      if(Cpp::EnvironmentManager::isSimplifiedMatching()) {
         //Make sure that proxy-contexts and content-contexts never have the same identity, even if they have the same content.
-            m_firstEnvironmentFile->setIdentityOffset(1); //Mark the first environment-file as the proxy
-            IndexedString u = parentJob()->document();
-            m_secondEnvironmentFile = new Cpp::EnvironmentFile(  u, 0 );
-        }
+        m_firstEnvironmentFile->setIdentityOffset(1); //Mark the first environment-file as the proxy
+        IndexedString u = parentJob()->document();
+        m_secondEnvironmentFile = new Cpp::EnvironmentFile(  u, 0 );
+      }
     }
 
     rpp::pp preprocessor(this);
@@ -132,6 +151,8 @@ void PreprocessJob::run()
         delete standardEnv;
     }
     
+    Cpp::ReferenceCountedStringSet macroNamesAtBeginning = m_currentEnvironment->macroNameSet();
+    
     KDevelop::ParsingEnvironmentFilePointer updatingEnvironmentFile;
     
     {
@@ -158,10 +179,8 @@ void PreprocessJob::run()
                 readLock.lock();
                 
                 needsUpdate = CppLanguageSupport::self()->needsUpdate(Cpp::EnvironmentFilePointer(cppEnv), localPath, includePaths);
-                
               }
             
-                  
             if(!needsUpdate) {
               parentJob()->setNeedsUpdate(false);
               return;
@@ -181,6 +200,7 @@ void PreprocessJob::run()
 
     QByteArray contents;
 
+    
     QString localFile(parentJob()->document().toUrl().toLocalFile());
   
     QFileInfo fileInfo( localFile );
@@ -191,7 +211,7 @@ void PreprocessJob::run()
         if ( !file.open( QIODevice::ReadOnly ) )
         {
             KDevelop::ProblemPointer p(new Problem());
-            p->setSource(KDevelop::Problem::Disk);
+            p->setSource(KDevelop::ProblemData::Disk);
             p->setDescription(i18n( "Could not open file '%1'", localFile ));
             switch (file.error()) {
               case QFile::ReadError:
@@ -233,11 +253,6 @@ void PreprocessJob::run()
         m_firstEnvironmentFile->setModificationRevision( KDevelop::ModificationRevision( fileInfo.lastModified(), parentJob()->revisionToken() ) );
     }
     
-    KDevelop::ModificationRevision::clearModificationCache(parentJob()->document());
-
-    if(m_secondEnvironmentFile) //Copy some information from the environment-file to its content-part
-        m_secondEnvironmentFile->setModificationRevision(m_firstEnvironmentFile->modificationRevision());
-
     ifDebug( kDebug( 9007 ) << "===-- PREPROCESSING --===> "
     << parentJob()->document().str()
     << "<== readFromDisk:" << readFromDisk
@@ -246,6 +261,7 @@ void PreprocessJob::run()
 
     if (checkAbort())
         return;
+    
 
     // Create a new macro block if this is the master preprocess-job
     if( parentJob()->masterJob() == parentJob() )
@@ -280,29 +296,48 @@ void PreprocessJob::run()
 
     PreprocessedContents result = preprocessor.processFile(parentJob()->document().str(), contents);
 
-    m_currentEnvironment->finish();
+    if(Cpp::EnvironmentManager::matchingLevel() <= Cpp::EnvironmentManager::Naive && !m_headerSectionEnded && !m_firstEnvironmentFile->headerGuard().isEmpty()) {
+      if(macroNamesAtBeginning.contains(m_firstEnvironmentFile->headerGuard())) {
+        //Remove the header-guard, and re-preprocess, since we don't do real environment-management(We don't allow empty versions)
+        m_currentEnvironment->removeMacro(m_firstEnvironmentFile->headerGuard());
+        result = preprocessor.processFile(parentJob()->document().str(), contents);
+      }
+    }
     
     if(!m_headerSectionEnded) {
       ifDebug( kDebug(9007) << parentJob()->document().str() << ": header-section was not ended"; )
       headerSectionEndedInternal(0);
     }
     
+    m_currentEnvironment->finishEnvironment(m_currentEnvironment->environmentFile() == m_updatingEnvironmentFile);
+    
     foreach (KDevelop::ProblemPointer p, preprocessor.problems()) {
       p->setLocationStack(parentJob()->includeStack());
-      p->setSource(KDevelop::Problem::Preprocessor);
+      p->setSource(KDevelop::ProblemData::Preprocessor);
       parentJob()->addPreprocessorProblem(p);
     }
 
     parentJob()->parseSession()->setContents( result, m_currentEnvironment->takeLocationTable() );
     parentJob()->parseSession()->setUrl( parentJob()->document() );
 
+    
     if(m_secondEnvironmentFile)
       parentJob()->setProxyEnvironmentFile( m_firstEnvironmentFile.data() );
     else
       parentJob()->setContentEnvironmentFile( m_firstEnvironmentFile.data() );
-
+    
+    if(m_secondEnvironmentFile) {//Copy some information from the environment-file to its content-part
+        KDevelop::DUChainWriteLocker readLock(KDevelop::DUChain::lock());
+        m_secondEnvironmentFile->setModificationRevision(m_firstEnvironmentFile->modificationRevision());
+        if(m_firstEnvironmentFile->headerGuard().isEmpty())
+          m_firstEnvironmentFile->setHeaderGuard(m_secondEnvironmentFile->headerGuard());
+        else
+          m_secondEnvironmentFile->setHeaderGuard(m_firstEnvironmentFile->headerGuard());
+    }
+    
     if( m_secondEnvironmentFile ) {
         //kDebug(9008) << parentJob()->document().str() << "Merging content-environment file into header environment-file";
+        KDevelop::DUChainReadLocker readLock(KDevelop::DUChain::lock());
         m_firstEnvironmentFile->merge(*m_secondEnvironmentFile);
         parentJob()->setContentEnvironmentFile(m_secondEnvironmentFile.data());
     }
@@ -311,6 +346,7 @@ void PreprocessJob::run()
         //If we are included from another preprocessor, give it back the modified macros,
         parentPreprocessor->m_currentEnvironment->swapMacros( m_currentEnvironment );
         //Merge include-file-set, defined macros, used macros, and string-set
+        KDevelop::DUChainReadLocker readLock(KDevelop::DUChain::lock());
         parentPreprocessor->m_currentEnvironment->environmentFile()->merge(*m_firstEnvironmentFile);
     }else{
 /*        kDebug(9007) << "Macros:";
@@ -395,6 +431,7 @@ void PreprocessJob::headerSectionEndedInternal(rpp::Stream* stream)
             if(contentEnvironment->matchEnvironment(m_currentEnvironment) && !CppLanguageSupport::self()->needsUpdate(contentEnvironment, localPath, parentJob()->includePathUrls()) && (!parentJob()->masterJob()->needUpdateEverything() || parentJob()->masterJob()->wasUpdated(content)) && (content->parsingEnvironmentFile()->featuresSatisfied(parentJob()->minimumFeatures())) ) {
                 //We can completely re-use the specialized context:
                 m_secondEnvironmentFile = dynamic_cast<Cpp::EnvironmentFile*>(content->parsingEnvironmentFile().data());
+                m_updatingEnvironmentFile = m_secondEnvironmentFile;
                 
                 //Merge the macros etc. into the current environment
                 m_currentEnvironment->merge( m_secondEnvironmentFile.data() );
@@ -414,7 +451,7 @@ void PreprocessJob::headerSectionEndedInternal(rpp::Stream* stream)
             ifDebug( kDebug(9007) << "could not find a matching content-context"; )
         }
 
-        m_currentEnvironment->finish();
+        m_currentEnvironment->finishEnvironment();
 
         m_currentEnvironment->setEnvironmentFile(m_secondEnvironmentFile);
     }
@@ -428,9 +465,29 @@ void PreprocessJob::headerSectionEndedInternal(rpp::Stream* stream)
 rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, int sourceLine, bool skipCurrentPath)
 {
     Q_UNUSED(type)
+    if(0){
+      uint currentDepth = 0;
+      CPPParseJob* job = parentJob();
+      while(job->parentPreprocessor()) {
+        ++currentDepth;
+        job = job->parentPreprocessor()->parentJob();
+      }
+      if(currentDepth > maxIncludeDepth) {
+        kDebug(9007) << "maximum depth reached while including" << _fileName << "into" << parentJob()->document().str();
+        return 0;
+      }
+    }
+    
     
     KUrl fileNameUrl(_fileName);
     
+    TopDUContext::Features slaveMinimumFeatures = TopDUContext::VisibleDeclarationsAndContexts;
+    if((parentJob()->minimumFeatures() & TopDUContext::AllDeclarationsContextsAndUsesForRecursive) == TopDUContext::AllDeclarationsContextsAndUsesForRecursive)
+      slaveMinimumFeatures = parentJob()->minimumFeatures();
+    
+    if(parentJob()->minimumFeatures() & TopDUContext::ForceUpdateRecursive)
+      slaveMinimumFeatures = (TopDUContext::Features)(slaveMinimumFeatures | TopDUContext::ForceUpdateRecursive);
+    
     QString fileName = fileNameUrl.pathOrUrl();
     
     if (checkAbort())
@@ -450,14 +507,15 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
     KUrl includedFile = included.first;
     if (includedFile.isValid()) {
       
+        IndexedString indexedFile(includedFile);
+        
         {
           //Prevent recursion that may cause a crash
-          IndexedString indexedFile(includedFile);
           PreprocessJob* current = this;
           while(current) {
             if(current->parentJob()->document() == indexedFile) {
               KDevelop::ProblemPointer p(new Problem()); ///@todo create special include-problem
-              p->setSource(KDevelop::Problem::Preprocessor);
+              p->setSource(KDevelop::ProblemData::Preprocessor);
               p->setDescription(i18n("File was included recursively from within itself: %1", fileName ));
               p->setFinalLocation(DocumentRange(parentJob()->document().str(), KTextEditor::Cursor(sourceLine,0), KTextEditor::Cursor(sourceLine+1,0)));
               p->setLocationStack(parentJob()->includeStack());
@@ -468,29 +526,65 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
           }
         }
       
-        if( m_updatingEnvironmentFile && m_updatingEnvironmentFile->missingIncludeFiles().contains(IndexedString(fileName)) ) {
-          //We are finding a file that was not in the include-path last time
-          //All following contexts need to be updated, because they may contain references to missing declarations
-          parentJob()->masterJob()->setNeedUpdateEverything( true );
-          kDebug(9007) << "Marking every following encountered context to be updated";
-        }
-
         ifDebug( kDebug(9007) << "PreprocessJob" << parentJob()->document().str() << "(" << m_currentEnvironment->environment().size() << "macros)" << ": found include-file" << fileName << ":" << includedFile; )
 
         KDevelop::ReferencedTopDUContext includedContext;
         bool updateNeeded = false;
+        bool updateForbidden = false;
 
         {
             KDevelop::DUChainReadLocker readLock(KDevelop::DUChain::lock());
             includedContext = KDevelop::DUChain::self()->chainForDocument(includedFile, m_currentEnvironment, (bool)m_secondEnvironmentFile);
+            
+            //Check if the same file is being processed by one of the parents, and if it is, import it later on
+            if(Cpp::EnvironmentManager::matchingLevel() <= Cpp::EnvironmentManager::Naive) {
+              
+              CPPParseJob* job = parentJob();
+              while(job->parentPreprocessor()) {
+                job = job->parentPreprocessor()->parentJob();
+                if(job->document() == indexedFile) {
+                  parentJob()->addDelayedImport(CPPParseJob::LineJobPair(job, sourceLine));
+                  return 0;
+                }
+              }
+              
+//               if(!includedContext) {
+//                 //Check if there is a parsed version that is disabled by its header-guard right now, and enventually use that one.
+//                 QList<ParsingEnvironmentFilePointer> allVersions = DUChain::self()->allEnvironmentFiles(indexedFile);
+//                 foreach(ParsingEnvironmentFilePointer version, allVersions) {
+//                   Cpp::EnvironmentFile* envFile = dynamic_cast<Cpp::EnvironmentFile*>(version.data());
+//                   
+//                   if(envFile && (envFile->isProxyContext() || !m_secondEnvironmentFile) && !envFile->headerGuard().isEmpty()) {
+//                     if(m_currentEnvironment->macroNameSet().contains(envFile->headerGuard())) {
+//                       includedContext = envFile->topContext();
+//                       
+//                       break;
+//                     }
+//                   }
+//                 }
+//               }
+            }
+            
             if(includedContext) {
               Cpp::EnvironmentFilePointer includedEnvironment(dynamic_cast<Cpp::EnvironmentFile*>(includedContext->parsingEnvironmentFile().data()));
-              if( includedEnvironment )
-                updateNeeded = CppLanguageSupport::self()->needsUpdate(includedEnvironment, localPath, parentJob()->includePathUrls()) || !includedEnvironment->featuresSatisfied(parentJob()->minimumFeatures());
+              if( includedEnvironment ) {
+                updateNeeded = CppLanguageSupport::self()->needsUpdate(includedEnvironment, localPath, parentJob()->includePathUrls());
+                updateNeeded |= !includedEnvironment->featuresSatisfied((TopDUContext::Features)(slaveMinimumFeatures & (~TopDUContext::ForceUpdateRecursive)));
+                //Do not update again if ForceUpdate is given and the context was already updated during this run
+                updateNeeded |= (slaveMinimumFeatures & TopDUContext::ForceUpdate) && !parentJob()->masterJob()->wasUpdated(includedContext.data());
+                
+                ///NEVER update when the file is header-guarded in the current environment, because else it will be emptied
+                if(!includedEnvironment->headerGuard().isEmpty() && m_currentEnvironment->macroNameSet().contains(includedEnvironment->headerGuard())) {
+                  updateForbidden = true;
+//                   kDebug() << "forbidding update of" << includedFile;
+                  updateNeeded = false;
+                }
+              }
             }
         }
 
-        if( includedContext && !updateNeeded && (!parentJob()->masterJob()->needUpdateEverything() || parentJob()->masterJob()->wasUpdated(includedContext)) ) {
+        if( includedContext && (updateForbidden ||
+                               (!updateNeeded && (!parentJob()->masterJob()->needUpdateEverything() || parentJob()->masterJob()->wasUpdated(includedContext))) )) {
             ifDebug( kDebug(9007) << "PreprocessJob" << parentJob()->document().str() << ": took included file from the du-chain" << fileName; )
 
             KDevelop::DUChainReadLocker readLock(KDevelop::DUChain::lock());
@@ -505,9 +599,10 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
                 ifDebug( kDebug(9007) << "preprocessjob: included file" << includedFile << "found in du-chain, but it has no parse-environment information, or it was not parsed by c++ support"; )
             }
         } else {
+            Q_ASSERT(!updateForbidden);
             if(updateNeeded)
               kDebug(9007) << "PreprocessJob" << parentJob()->document().str() << ": need to update" << includedFile;
-            else if(parentJob()->masterJob()->needUpdateEverything())
+            else if(parentJob()->masterJob()->needUpdateEverything() && includedContext)
               kDebug(9007) << "PreprocessJob" << parentJob()->document().str() << ": needUpateEverything, updating" << includedFile;
             else
               kDebug(9007) << "PreprocessJob" << parentJob()->document().str() << ": no fitting entry for" << includedFile << "in du-chain, parsing";
@@ -527,9 +622,9 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
 
             ///The second parameter is zero because we are in a background-thread and we here
             ///cannot create a slave of the foreground cpp-support-part.
-            CPPParseJob* slaveJob = new CPPParseJob(includedFile, 0, this);
-            if((parentJob()->minimumFeatures() & TopDUContext::AllDeclarationsContextsAndUsesForRecursive) == TopDUContext::AllDeclarationsContextsAndUsesForRecursive)
-              slaveJob->setMinimumFeatures(parentJob()->minimumFeatures());
+            CPPParseJob* slaveJob = new CPPParseJob(includedFile, this);
+            
+            slaveJob->setMinimumFeatures(slaveMinimumFeatures);
 
             slaveJob->setIncludedFromPath(included.second);
 
@@ -545,10 +640,21 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
             delete slaveJob;
         }
         ifDebug( kDebug(9007) << "PreprocessJob" << parentJob()->document().str() << "(" << m_currentEnvironment->environment().size() << "macros)" << ": file included"; )
+    
+        {
+          DUChainReadLocker lock(DUChain::lock());
+          if( m_updatingEnvironmentFile && m_updatingEnvironmentFile->missingIncludeFiles().contains(IndexedString(fileName)) ) {
+            //We are finding a file that was not in the include-path last time
+            //All following contexts need to be updated, because they may contain references to missing declarations
+            parentJob()->masterJob()->setNeedUpdateEverything( true );
+            kDebug(9007) << "Marking every following encountered context to be updated";
+          }
+        }
+    
     } else {
         kDebug(9007) << "PreprocessJob" << parentJob()->document().str() << ": include not found:" << fileName;
         KDevelop::ProblemPointer p(new Problem()); ///@todo create special include-problem
-        p->setSource(KDevelop::Problem::Preprocessor);
+        p->setSource(KDevelop::ProblemData::Preprocessor);
         p->setDescription(i18n("Included file was not found: %1", fileName ));
         p->setExplanation(i18n("Searched include path:\n%1", urlsToString(parentJob()->includePathUrls())));
         p->setFinalLocation(DocumentRange(parentJob()->document().str(), KTextEditor::Cursor(sourceLine,0), KTextEditor::Cursor(sourceLine+1,0)));
@@ -559,7 +665,7 @@ rpp::Stream* PreprocessJob::sourceNeeded(QString& _fileName, IncludeType type, i
         ///Before doing that, model findInclude(..) exactly after the standard
         m_firstEnvironmentFile->addMissingIncludeFile(IndexedString(fileName));
     }
-
+    
     return 0;
 }
 
