@@ -204,6 +204,43 @@ void TestCppCodeCompletion::testPrivateVariableCompletion() {
   release(context);
 }
 
+void TestCppCodeCompletion::testSameNamespace() {
+  TEST_FILE_PARSE_ONLY
+
+  //                 0         1         2         3         4         5         6         7
+  //                 0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012
+  QByteArray method("namespace A { class C { }; void test2(); } namespace A { void test() { } }");
+
+  TopDUContext* top = parse(method, DumpNone);
+
+  DUChainWriteLocker lock(DUChain::lock());
+
+  QVERIFY(!top->parentContext());
+  QCOMPARE(top->childContexts().count(), 2);
+  QCOMPARE(top->childContexts()[1]->childContexts().count(), 2);
+
+//   lock.unlock();
+  {
+    Cpp::CodeCompletionContext::Ptr cptr( new  Cpp::CodeCompletionContext(DUContextPointer(top->childContexts()[1]->childContexts()[1]), "; ", QString()) );
+    bool abort = false;
+    typedef KSharedPtr <KDevelop::CompletionTreeItem > Item;
+    
+    QList <Item > items = cptr->completionItems(top->range().end, abort);
+    QStandardItemModel fakeModel;
+    foreach(Item i, items) {
+      NormalDeclarationCompletionItem* decItem  = dynamic_cast<NormalDeclarationCompletionItem*>(i.data());
+      QVERIFY(decItem);
+      kDebug() << decItem->declaration()->toString();
+      kDebug() << i->data(fakeModel.index(0, KTextEditor::CodeCompletionModel::Name), Qt::DisplayRole, 0).toString();
+    }
+    
+    //Have been filtered out, because only types are shown from the global scope
+    QCOMPARE(items.count(), 3); //C, test2, test
+  }
+  
+  release(top);
+}
+
 void TestCppCodeCompletion::testUnnamedNamespace() {
   TEST_FILE_PARSE_ONLY
 
