@@ -1,6 +1,6 @@
 /* KDevelop xUnit plugin
  *
- * Copyright 2008 Manuel Breugelmans <mbr.nxi@gmail.com>
+ * Copyright 2008-2009 Manuel Breugelmans <mbr.nxi@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -38,8 +38,9 @@ class ProjectFolderItem;
 class IProject;
 }
 
-namespace Veritas { class Test; }
+namespace Veritas { class Test; class TestToolViewFactory; }
 namespace QTest { class KDevRegister; }
+class KAction;
 
 /*! Makes the QTestRunner toolview available */
 class QTestPlugin : public KDevelop::IPlugin, public Veritas::ITestFramework
@@ -56,14 +57,18 @@ public:
 
     virtual QWidget* createConfigWidget();
     virtual KDevelop::ProjectConfigSkeleton* configSkeleton(const QVariantList& args);
-
+    virtual void unload();
+    
 private slots:
     void newQTest();
+    void showNewTestWizard();
 
 private:
     KDevelop::ProjectFolderItem* m_dir;
     QTestOutputDelegate* m_delegate;
     KDevelop::IProject* m_proj; // scratch variable
+    Veritas::TestToolViewFactory* m_toolFactory;
+    KAction* m_newQTestAction;
 };
 
 #endif // QTEST_QTESTPLUGIN_H
